/*
 * Channel.java
 *
 * This code is released to the public domain.
 *
 * Created by Mattias Larsson, ml@techno.org
 *
 * Created on April 4, 2003, 4:42 PM
 */
/*
 * Modified by Lester Jacobs, April 28th, 2003
 * Changes made to extract the tuner channel
 * and station call signs from the xmltv data.
 * Search on: lester1
 *
 * Modified by Lester Jacobs, July 1st, 2003
 * Changes made to account for channel id changes
 * in new xmltv output.
 * Search on: lester2
 */

package xmltv;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.io.PrintWriter;

/**
 * A channel e.g. MTV or CNN.
 *
 * A channel has the following values:
 * <bl>
 * <li>name - the normal short version, e.g. CNN, MTV.</li>
 * <li>longName - a descriptive version of the name, e.g. Cable News Network, Music Television.</li>
 * <li>network - the parent network, e.g. BBC, ABC.</li>
 * <li>stationID - a GUID for the station.</li>
 * <li>shows - a map of Programme objects containing the shows for that channel.</li>
 * </bl>
 *
 * @author  mta
 * @author  Steinar Hansen <mailto: sh@broadpark.no>
 */
public class Channel {

    /** used for more verbose output*/
    static boolean debug = false;

    private String id;
    private String name;
    private String longname;
    private String network;
    private int stationID;
    private Map id2programme = new HashMap();

    private PrintWriter pw;

    public Channel(String id, PrintWriter pw){
        this.id = id;
        this.pw = pw;
        this.stationID = id.hashCode();

        if(debug){
            XMLTVImportPlugin.printlnWithPrependedDate(this.pw, "Channel created, id="+id);
        }
    }

    public String getId() {
        return id;
    }

    public int getStationID() {
        return stationID;
    }

    public void setName(String name) {
        this.name = name;
        this.longname = "";
        this.network = "";
    }

    public void addProgramme(String id, Programme programme){
        id2programme.put(id, programme);
    }

    public Programme getProgramme(String id){
        return (Programme) id2programme.get(id);
    }

    public void insertIntoEPG(sage.EPGDBPublic epg) {
        if(debug){
            XMLTVImportPlugin.printlnWithPrependedDate(pw,
                    "Channel.insertIntoEPG start, name: "+this.name+
                    ", longname: "+this.longname+
                    ", network: "+this.network+
                    ", stationID: "+this.stationID);
        }
        epg.addChannelPublic(this.name, this.longname, this.network, this.stationID);
        Iterator it = this.id2programme.keySet().iterator();
        while (it.hasNext()) {
            Programme programme = (Programme) this.id2programme.get(it.next());
            if(programme!=null){
                programme.insertIntoEPG(epg);
            }
        }
        if(debug){
            XMLTVImportPlugin.printlnWithPrependedDate(pw, "Channel.insertIntoEPG end, name: "+this.name);
        }
    }

}

