/*
 * Programme.java
 *
 * This code is released to the public domain.
 *
 * Created by Mattias Larsson, ml@techno.org
 *
 * Created on April 22, 2003, 5:31 PM
 *
 * Modified by Lester Jacobs, April 28th, 2003
 * Code changed to detect rerun indicator
 * (<previously-shown> tag) in xmltv data
 * Search on: lester1
 *
 * Modified by Steinar Hansen, July 29th 2004
 * Major rewrite, Show renamed to Programme since that is what it is called in
 * the xmltv output.
 */

package xmltv;

import java.util.zip.CRC32;
import java.util.Locale;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;
import java.io.PrintWriter;

/**
 * A programme defines a tv-show with a start-time and end-time.
 *
 * For the XMLTVImportPlugin to run a programme must have startTime <code>setStart()</code>
 * and endTime <code>setEnd()</code> 
 *
 * @author  mta
 * @author  Steinar Hansen <mailto: sh@broadpark.no>
 */
public class Programme implements Cloneable{

    /** used for more verbose output*/
    static boolean debug = false;

    private String title;
    private String desc;
    private int duration;
    private String subtitles;

    private String extId;
    private boolean rerun;

    private Locale locale;

    private String startString;
    private String endString;

    private PrintWriter pw;
    private Channel channel;

    public long start;
    public long length;
    
    public int chanId;
    


    /** Creates a new instance of Programme */
    public Programme(Locale locale, PrintWriter pw, Channel channel) {

        this.locale = locale;
        this.pw = pw;
        this.channel = channel;

        if(debug){
            XMLTVImportPlugin.printlnWithPrependedDate(this.pw, "Programme created, channel="+channel.getId());
        }
    }

    public Locale getLocale() {
        return locale;
    }

    /**
     * The id is calculated from the hashcode of the startString + hashcode
     * of the endString.
     *
     * We should be fairly safe in assuming that we will not get two programmes that
     * starts and ends at the same time on the same channel.
     * @return unique id for this programme.
     */
    public String getId(){
        int result;
        result = (startString != null ? startString.hashCode() : 0);
        result = 29 * result + (endString != null ? endString.hashCode() : 0);

        return ""+result;
    }

    /**
     * Sets the startTime of the programme from a String that must pass
     * @param s
     */
    public void setStart(String s) {
        if (s == null)
            return;
        this.startString = s;
        try {
            if (s.indexOf(" ") == 12) {
                this.start = XMLTVImportPlugin.formatterNoSeconds.parse(s).getTime();
            } else {
                this.start = XMLTVImportPlugin.formatterSeconds.parse(s).getTime();
            }
        } catch (java.text.ParseException e) {
            System.out.println("err: "+e.toString());
            e.printStackTrace();
            XMLTVImportPlugin.printlnWithPrependedDate(pw, "ParseException");
            e.printStackTrace(pw);
            pw.flush();
        }
    }
    
    public void setEnd(String s) {
        if (s == null)
            return;
        this.endString = s;
        try {
            if (s.indexOf(" ") == 12) {
                this.length = XMLTVImportPlugin.formatterNoSeconds.parse(s).getTime()-this.start;
            } else {
                this.length = XMLTVImportPlugin.formatterSeconds.parse(s).getTime()-this.start;
            }
            this.duration = (int)this.length;
        } catch (java.text.ParseException e) {
            System.out.println("err: "+e.toString());
            e.printStackTrace();
            XMLTVImportPlugin.printlnWithPrependedDate(pw, "ParseException");
            e.printStackTrace(pw);
        }
    }
    
    public void setTitle(String s) {
        this.title = s;
    }
    
    public void setDesc(String s) {
        this.desc = s;
    }

    public void setRerun(boolean isRerun) {
        this.rerun = isRerun;
    }
    
    public void setSubtitles(String subtitles) {
        this.subtitles = subtitles;
    }

    public void insertIntoEPG(sage.EPGDBPublic epg) {
        if(debug){
            XMLTVImportPlugin.printlnWithPrependedDate(pw, "Programme.insertIntoEPG start, title="+title);
        }
        // create extId
        createExtId();

        if (!rerun) { // firstrun
            if (!epg.addShowPublic(null, this.title, null, this.desc, this.duration,
            null, null, null, null, null,null, null, null, null,
            this.extId, this.locale.getLanguage(), 0)) {
                XMLTVImportPlugin.printlnWithPrependedDate(pw, "addShowPublic for title: "+this.title+" failed");
            }else{
                if(debug){
                    XMLTVImportPlugin.printlnWithPrependedDate(pw,
                            "addShowPublic for title: "+this.title+
                            ", desc: "+this.desc+
                            ", duration: "+getDurationAsHoursAndMinutes(this.duration)+
                            ", extId: "+this.extId+
                            ", language: "+this.locale.getLanguage());
                }
            }
        } else { // rerun
            if (!epg.addShowPublic(this.title, null, null, this.desc, this.duration,
            null, null, null, null, null,null, null, null, null,
            this.extId, this.locale.getLanguage(), 0)) {
                XMLTVImportPlugin.printlnWithPrependedDate(pw, "addShowPublic for rerun title: "+this.title+" failed");
            }else{
                if(debug){
                    XMLTVImportPlugin.printlnWithPrependedDate(pw,
                            "addShowPublic for rerun title: "+this.title+
                            ", desc: "+this.desc+
                            ", duration: "+getDurationAsHoursAndMinutes(this.duration)+
                            ", extId: "+this.extId+
                            ", language: "+this.locale.getLanguage());
                }
            }
        }

        if(this.duration>0){
            if (!epg.addAiringPublic(this.extId, this.channel.getStationID(), this.start, this.duration)) {
                XMLTVImportPlugin.printlnWithPrependedDate(pw, "addAiringPublic for title: "+this.title+" failed");
            }else{
                if(debug){
                    XMLTVImportPlugin.printlnWithPrependedDate(pw,
                            "addAiringPublic for title: "+this.title+
                            ", extId: "+this.extId+
                            ", stationID: "+this.channel.getStationID()+
                            ", start: "+getTimeAsText(this.start)+
                            ", duration: "+getDurationAsHoursAndMinutes(this.duration));
                }
            }
        }else{
            XMLTVImportPlugin.printlnWithPrependedDate(pw, "airing on channel: "+channel.getId()+", with title: "+title+" has a duration of 0, skipping.");
        }
        if(debug){
            XMLTVImportPlugin.printlnWithPrependedDate(pw, "Programme.insertIntoEPG end, title="+title);
        }
    }

    private void createExtId() {
        CRC32 crc = new CRC32();
        if (this.title != null){
            crc.update(this.title.getBytes());
        }
        if (this.startString != null) {
            crc.update(this.startString.getBytes());
        }
        if (this.desc != null){
            crc.update(this.desc.getBytes());
        }
        this.extId = ""+crc.getValue();
    }

    private String getTimeAsText(long time){
        SimpleDateFormat formatter = new SimpleDateFormat("dd.MM.yyyy HH:mm");
        Date date = new Date(time);
        return formatter.format(date);
    }

    private String getDurationAsHoursAndMinutes(long duration){
        long seconds = duration/1000;//7200
        long minutes = seconds/60;//120
                                  //90
                                  //35
        long hours = minutes/60;
        long minutes_remainder = minutes%60;
        DecimalFormat twoDigitFormat = new DecimalFormat("00");
        return twoDigitFormat.format(hours)+":"+twoDigitFormat.format(minutes_remainder) ;

    }
}
