package xmltv;

import org.xml.sax.*;

import java.io.PrintWriter;
import java.util.Map;
import java.util.Locale;

/**
 * This is the callback point for the SAXParser.
 *
 * @author  Steinar Hansen <mailto: sh@broadpark.no>
 */
public class XMLParserCallback implements ContentHandler, ErrorHandler{

    /** used for more verbose output*/
    static boolean debug = false;
    
    private PrintWriter pw;
    private StringBuffer elementContentBuffer = new StringBuffer();

    private boolean isWrongLanguageSoSkip = false;

    private String channelId;
    private String programmeId;
    private Map id2Channel;

    public XMLParserCallback(PrintWriter pw, Map id2Channel){
        this.pw = pw;
        this.id2Channel = id2Channel;
    }

    public void endDocument() throws SAXException {
    }

    public void startDocument() throws SAXException {
    }

    /**
     * This is called when parsing elements that contains only text.
     *
     * e.g. <test>hello there</test> will cause this to be called with
     * 'hello there' in ch.
     * @param ch
     * @param start
     * @param length
     * @throws SAXException
     */
    public void characters(char ch[], int start, int length) throws SAXException {
        //This stores temporarilly the element contents, e.g. display-name and title.
        if(this.elementContentBuffer!=null){
            this.elementContentBuffer.append(ch, start, length);
            if(debug){
                pw.println(this.elementContentBuffer.toString());
            }
        }
    }

    public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
    }

    public void endPrefixMapping(String prefix) throws SAXException {
    }

    public void skippedEntity(String name) throws SAXException {
    }

    public void setDocumentLocator(Locator locator) {
    }

    public void processingInstruction(String target, String data) throws SAXException {
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }

    /**
     * Called when parsing a start element, e.g. <channel>
     *
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param atts
     * @throws SAXException
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        String tag = qName.toLowerCase();
        if (tag.equals("channel")) {
            channelId = atts.getValue("id");
            id2Channel.put(channelId, new Channel(channelId, pw));
        } else if (tag.equals("display-name")) {
            //Nothing to do here, the setting of values is done in the endElement method.
        } else if (tag.equals("programme")) {
            channelId = atts.getValue("channel");
            Channel channel = (Channel) id2Channel.get(channelId);
            Programme programme = new Programme(Locale.getDefault(), pw, (Channel) id2Channel.get(channelId));
            programme.setStart(atts.getValue("start"));
            programme.setEnd(atts.getValue("stop"));
            channel.addProgramme(programme.getId(), programme);
            programmeId = programme.getId();
        } else if (tag.equals("title")){
            Programme programme = ((Channel)id2Channel.get(channelId)).getProgramme(programmeId);
            if(atts.getValue("lang")!=null&&!atts.getValue("lang").equals(programme.getLocale().getLanguage())){
                isWrongLanguageSoSkip = true;
            }
        } else if (tag.equals("desc")){
            Programme programme = ((Channel)id2Channel.get(channelId)).getProgramme(programmeId);
            if(atts.getValue("lang")!=null&&!atts.getValue("lang").equals(programme.getLocale().getLanguage())){
                isWrongLanguageSoSkip = true;
            }
        } else if (tag.equals("subtitles")){
            Programme programme = ((Channel)id2Channel.get(channelId)).getProgramme(programmeId);
            programme.setSubtitles(atts.getValue("type"));
        }else if (tag.equals("previously-shown")){
            Programme programme = ((Channel)id2Channel.get(channelId)).getProgramme(programmeId);
            programme.setRerun(true);
        }
    }

    /**
     * Called when parsing an end element, e.g. </channel>.
     *
     * @param namespaceURI
     * @param localName
     * @param qName
     * @throws SAXException
     */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        String tag = qName.toLowerCase();
        if (tag.equals("channel")) {
            resetElementContentBuffer();
            channelId = null;
        } else if (tag.equals("display-name")) {
            Channel channel = (Channel) id2Channel.get(channelId);
            channel.setName(elementContentBuffer.toString());
            resetElementContentBuffer();
        } else if (tag.equals("programme")) {
            // add programme here...
            resetElementContentBuffer();
            programmeId = null;
        } else if (tag.equals("desc")) {
            if(!isWrongLanguageSoSkip){
                Programme programme = ((Channel)id2Channel.get(channelId)).getProgramme(programmeId);
                programme.setDesc(elementContentBuffer.toString());
            }
            resetElementContentBuffer();
            isWrongLanguageSoSkip = false;
        } else if (tag.equals("title")) {
            if(!isWrongLanguageSoSkip){
                Programme programme = ((Channel)id2Channel.get(channelId)).getProgramme(programmeId);
                programme.setTitle(elementContentBuffer.toString());
            }
            resetElementContentBuffer();
            isWrongLanguageSoSkip = false;
        }
    }

    //==================================================================
    //  ERROR HANDLING
    //==================================================================

    public void error(SAXParseException exception) throws SAXException {
        XMLTVImportPlugin.printlnWithPrependedDate(pw, "Error");
        XMLTVImportPlugin.printlnWithPrependedDate(pw, "line number: "+exception.getLineNumber()+", column number: "+exception.getColumnNumber());
        exception.printStackTrace(pw);
    }

    public void fatalError(SAXParseException exception) throws SAXException {
        XMLTVImportPlugin.printlnWithPrependedDate(pw, "FatalError");
        XMLTVImportPlugin.printlnWithPrependedDate(pw, "line number: "+exception.getLineNumber()+", column number: "+exception.getColumnNumber());
        exception.printStackTrace(pw);
    }

    public void warning(SAXParseException exception) throws SAXException {
        XMLTVImportPlugin.printlnWithPrependedDate(pw, "Warning");
        XMLTVImportPlugin.printlnWithPrependedDate(pw, "line number: "+exception.getLineNumber()+", column number: "+exception.getColumnNumber());
        exception.printStackTrace(pw);
    }

    private void resetElementContentBuffer(){
        elementContentBuffer = new StringBuffer();
    }
}

