/*
 * XMLTVImportPlugin.java
 *
 * This code is released to the public domain.
 *
 * Created by Mattias Larsson, ml@techno.org
 *
 * Created on April 21, 2003, 5:52 PM
 *
 *
 * Modified by Lester Jacobs, April 28th, 2003
 * Code modified to extract the correct tuner channel
 * from the xmltv data.  Also reruns are now determined
 * from the <previously-shown> tag.
 * Search on: lester1
 *
 *
 * Modified by Steinar Hansen, July 29th, 2004.
 * Major rewrite since i didn't get the plugin to work. Now outputs
 * errormessages in xmltv_plugin.log and accepts properties from a
 * xmltv_plugin.properties.
 */

package xmltv;

import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.XMLReaderFactory;

import java.util.*;
import java.text.SimpleDateFormat;

/**
 * This plugin is called by reads a xml-file containing
 *
 * More information at:
 *
 * http://mta.techno.org/epg/
 *
 * @author  mta http://mta.techno.org/epg/
 * @author  Steinar Hansen <mailto: sh@broadpark.no>
 */
public class XMLTVImportPlugin implements sage.EPGImportPlugin{

    /** used for more verbose output*/
    static boolean debug = false;

    /** Default parser name. */
    protected static String parserName = "org.apache.xerces.parsers.SAXParser";

    /** Default date format. */
    protected static String dateFormatLogged = "dd.MMM.yyyy, HH:mm:ss";

    /** Default provider id. */
    protected static long providerId = 867507149L;

    /** Default provider name. */
    protected static String providerName = "XMLTV Lineup";

    public static String[][] prov
            = new String[][]{new String[]{ ""+providerId, providerName}};

    public static SimpleDateFormat formatterSeconds = new SimpleDateFormat("yyyyMMddHHmmss Z");

    public static SimpleDateFormat formatterNoSeconds = new SimpleDateFormat("yyyyMMddHHmm Z");

    /** Default xml filename. */
    protected static String epgDataFileName = "epgdata.xml";

    /** A map containing the channels and Programmes*/
    private HashMap channels;

    private static PrintWriter pw;
    private static Properties properties;
    private static SimpleDateFormat f = new SimpleDateFormat(dateFormatLogged);


    static{
        try {
            pw = new PrintWriter(new FileWriter("xmltv/xmltv_plugin.log"));
        } catch (IOException e) {
            e.printStackTrace(System.out);  //Not easy to report on this when PrintWriter hasn't been init yet.
        }
        try{
            pw.println("Initializing XMLTVImportPlugin");
            System.out.println("Initializing XMLTVImportPlugin");
            pw.println("working directory is: "+System.getProperty("user.dir"));
            System.out.println("working directory is: "+System.getProperty("user.dir"));

            properties = new Properties();
            printlnWithPrependedDate(pw, "loading properties");
            properties.load(new FileInputStream(new File("xmltv/xmltv.properties")));
            initProperties();
        } catch (Exception e) {
            e.printStackTrace(System.out);
            e.printStackTrace(pw);
        }finally{
            pw.flush();
        }
    }

    public static void printlnWithPrependedDate(PrintWriter pw, String s){
        pw.println("<"+f.format(new Date())+"> "+s);
        System.out.println("<"+f.format(new Date())+"> "+s);
    }

    /** Creates a new instance of XMLTVImportPlugin */
    public XMLTVImportPlugin() {
        this.channels = new HashMap();
    }

    private static void initProperties() {
        printlnWithPrependedDate(pw, "Initializing properties");
        if(properties.containsKey("SAXParser")){
            parserName = properties.getProperty("SAXParser");
            printlnWithPrependedDate(pw, "Found SAXParser="+parserName);
        }
        if(properties.containsKey("DateFormat")){
            dateFormatLogged = properties.getProperty("DateFormat");
            f = new SimpleDateFormat(dateFormatLogged);
            printlnWithPrependedDate(pw, "Found DateFormat="+f);
        }
        boolean reloadProvider = false;
        if(properties.containsKey("ProviderId")){
            providerId = Long.parseLong(properties.getProperty("ProviderId"));
            reloadProvider = true;
            printlnWithPrependedDate(pw, "Found ProviderId="+providerId);
        }
        if(properties.containsKey("ProviderName")){
            providerName = properties.getProperty("ProviderName");
            reloadProvider = true;
            printlnWithPrependedDate(pw, "Found ProviderName="+providerName);
        }
        if(reloadProvider){
            prov = new String[][]{new String[]{ ""+providerId, providerName }};
        }
        if(properties.containsKey("FormatterSeconds")){
            formatterSeconds = new SimpleDateFormat(properties.getProperty("FormatterSeconds"));
            printlnWithPrependedDate(pw, "Found FormatterSeconds="+formatterSeconds);
        }
        if(properties.containsKey("FormatterNoSeconds")){
            formatterNoSeconds = new SimpleDateFormat(properties.getProperty("FormatterNoSeconds"));
            printlnWithPrependedDate(pw, "Found FormatterNoSeconds="+formatterNoSeconds);
        }
        if(properties.containsKey("Debug")){
            debug =  (Boolean.valueOf(properties.getProperty("Debug"))).booleanValue();
            Channel.debug = debug;
            Programme.debug = debug;
            XMLParserCallback.debug = debug;
            printlnWithPrependedDate(pw, "Found Debug="+debug);
        }
        if(properties.containsKey("EpgDataFileName")){
            epgDataFileName =  properties.getProperty("EpgDataFileName");
            printlnWithPrependedDate(pw, "Found EpgDataFileName="+epgDataFileName);
        }
        printlnWithPrependedDate(pw, "Completed initializing properties");
    }

    public String[][] getLocalMarkets() {
        return prov;
    }
    
    public String[][] getProviders(String str) {
        return prov;
    }

    /**
     * The updateGuide method is what Sage is calling to refresh the EPG.
     *
     * @param str
     * @param epg
     * @return
     */
    public boolean updateGuide(String str, sage.EPGDBPublic epg) {
        printlnWithPrependedDate(pw, "updateGuide called");
        XMLReader parser = null;

        try {
            try{
                parser = XMLReaderFactory.createXMLReader(parserName);
            }
            catch (Exception e) {
                System.out.println("error: Unable to instantiate parser ("+parserName+")");
                printlnWithPrependedDate(pw, "error: Unable to instantiate parser ("+parserName+")");
                pw.flush();
            }

            parser.setFeature("http://xml.org/sax/features/validation", true);
            XMLParserCallback callback = new XMLParserCallback(pw, channels);
            parser.setContentHandler(callback);
            parser.setErrorHandler(callback);
            try {
                parser.parse(new InputSource(new FileInputStream(epgDataFileName)));
            } catch (SAXParseException e) {
                e.printStackTrace(System.out);
                printlnWithPrependedDate(pw, "line number: "+e.getLineNumber()+", column number: "+e.getColumnNumber());
                e.printStackTrace(pw);
                pw.flush();
                return false;
            }

            Map map = getChannel2TunerMap();

            epg.setLineup(providerId, map);
            Iterator iter = channels.values().iterator();
            while(iter.hasNext()){
                Channel channel = (Channel) iter.next();
                channel.insertIntoEPG(epg);
            }
            // provider hardcoded since sagetv doesnt call get providers
            
        } catch (Exception e) {
            e.printStackTrace(System.out);
            printlnWithPrependedDate(pw, "Error");
            e.printStackTrace(pw);
        } finally {
            pw.flush();
        }

        return true;
    }

    /**
     * We don't know what the tuner channels are, and the XML-file isn't telling us.
     * So just give a number from 1 and iterating to the number of channels.
     * @return Map containing stationID-># (from 1 to number of channels)
     */
    private Map getChannel2TunerMap() {
        if(debug){
            printlnWithPrependedDate(pw, "Start channel to tuner map.");
        }
        Map map = new HashMap();
        int tunerchan = 1;
        Iterator i = this.channels.values().iterator();
        while(i.hasNext()) {
            Channel c = (Channel)i.next();
            String ch = ""+tunerchan;
            tunerchan++;
            map.put(new Integer(c.getStationID()), new String[]{ ch });
            if(debug){
                printlnWithPrependedDate(pw, "stationID: "+c.getStationID()+", channel: "+ch);
            }
        }
        if(debug){
            printlnWithPrependedDate(pw, "End channel to tuner map.");
        }
        return map;
    }
}


